package jalview.io;

import jalview.gui.*;
import jalview.datamodel.*;
import jalview.schemes.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;


public class HTMLOutput
{
  AlignViewport av;
  SequenceRenderer sr;
  Color color;

  public HTMLOutput(AlignViewport av)
  {
    this.av = av;
    sr = new SequenceRenderer(av);
    JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty("LAST_DIRECTORY"));
    chooser.setFileView(new JalviewFileView());
    chooser.setDialogTitle("Save as HTML");
    chooser.setToolTipText("Save");
    int value = chooser.showSaveDialog(null);
    if(value == JalviewFileChooser.APPROVE_OPTION)
    {
      String choice =  chooser.getSelectedFile().getPath();
      jalview.bin.Cache.setProperty("LAST_DIRECTORY",  choice);
      try{
        PrintWriter out = new java.io.PrintWriter(new java.io.FileWriter(choice));
        out.println("<HTML>");
        out.println("<style type=\"text/css\">");
        out.println("<!--");
        out.print("td {font-family: \""+av.getFont().getFamily()+"\", \""+av.getFont().getName()+ "\", mono; "
                   +"font-size: "+av.getFont().getSize()+"px; ");


        if (av.getFont().getStyle() == Font.BOLD)
           out.print("font-weight: BOLD; ");

        if (av.getFont().getStyle() == Font.ITALIC)
           out.print("font-style: italic; ");

        out.println("text-align: center; }");

        out.println("-->");
        out.println("</style>");
        out.println("<BODY>");


        if(av.getWrapAlignment())
           drawWrappedAlignment(out);
        else
           drawUnwrappedAlignment(out);

        out.println("\n</body>\n</html>");
        out.close();
        jalview.util.BrowserLauncher.openURL( "file:///"+choice );
      }
      catch(Exception ex){ex.printStackTrace();}
    }
  }

  void drawUnwrappedAlignment(PrintWriter out)
  {

    out.println("<table border=\"1\"><tr><td>\n");
    out.println("<table border=\"0\"  cellpadding=\"0\" cellspacing=\"0\">\n");


    //////////////
    SequenceGroup [] groups;
    SequenceI seq;
    ColourSchemeI cs = null;
    AlignmentI alignment = av.getAlignment();
    String r,g,b;

  // draws the top row, the measure rule
    out.println("<tr><td colspan=\"6\"></td>");
    int i=0;
    for(i=10; i<alignment.getWidth()-10; i+=10)
      out.println("<td colspan=\"9\">"+i+"<br>|</td><td></td>");

    out.println("<td colspan=\"3\"></td><td colspan=\"3\">"+i+"<br>|</td>");
    out.println("</tr>");

    for (i = 0; i < alignment.getHeight(); i++)
    {
      seq = alignment.getSequenceAt(i);
      groups = alignment.findAllGroups( seq );
      out.println("<tr><td nowrap>"+seq.getDisplayId()+"&nbsp;&nbsp;</td>");


      for (int res = 0; res < seq.getLength(); res++)
      {
        cs = av.getGlobalColourScheme();

        if(groups!=null)
         {
           for (int k = 0; k < groups.length; k++)
             if (groups[k].getStartRes() <= res && groups[k].getEndRes() >= res)
             {
               cs = groups[k].cs;
               break;
             }
         }


        color = sr.getResidueBoxColour(cs, seq, res);

        if(color.getRGB()<-1)
        {
          r = Integer.toHexString(color.getRed());
          if(r.length()<2)
            r = "0"+r;
          g = Integer.toHexString(color.getGreen());
          if(g.length()<2)
            g = "0"+g;
          b = Integer.toHexString(color.getBlue());
          if(b.length()<2)
            b = "0"+b;
          out.println("<td bgcolor=\"#"+ r + g + b + "\">"+seq.getCharAt(res)+"</td>");
        }
        else
          out.println("<td>"+seq.getCharAt(res)+"</td>");

      }

      out.println("</tr>");
    }
    //////////////
    out.println("</table>");
    out.println("</td></tr></table>");

  }

  void drawWrappedAlignment(PrintWriter out)
  {
     ////////////////////////////////////
     /// How many sequences and residues can we fit on a printable page?
     AlignmentI al = av.getAlignment();
     SequenceGroup [] groups;
     SequenceI seq;
     ColourSchemeI cs = null;
     String r,g,b;

     out.println("<table border=\"1\"><tr><td>\n");
     out.println("<table border=\"0\"  cellpadding=\"0\" cellspacing=\"0\">\n");

     for(int startRes=0; startRes<al.getWidth(); startRes+=av.getChunkWidth())
     {

       int endRes = startRes+av.getChunkWidth();
       if(endRes>al.getWidth())
           endRes = al.getWidth();

     out.println("<tr>");
     out.println("<td colspan=\"6\">&nbsp;</td>");
     for (int i = startRes+10; i < endRes; i += 10)
       out.println("<td colspan=\"9\">" + i + "<br>|</td><td></td>");

    // out.println("<td colspan=\"3\"></td><td colspan=\"3\">" + i +
    //             "<br>|</td>");
     out.println("</tr>");



       for (int s = 0; s < al.getHeight(); s++)
       {
         out.println("<tr>");
         seq = al.getSequenceAt(s);
         groups = al.findAllGroups( seq );
         out.println("<td nowrap>"+seq.getDisplayId()+"&nbsp;&nbsp;</td>");

         for (int res = startRes; res < endRes; res++)
              {
                cs = av.getGlobalColourScheme();
                if(groups!=null)
                 {
                   for (int k = 0; k < groups.length; k++)
                     if (groups[k].getStartRes() <= res && groups[k].getEndRes() >= res)
                     {
                       cs = groups[k].cs;
                       break;
                     }
                 }

                color = sr.getResidueBoxColour(cs, seq, res);

                if(color.getRGB()<-1)
                {
                  r = Integer.toHexString(color.getRed());
                  if(r.length()<2)
                    r = "0"+r;
                  g = Integer.toHexString(color.getGreen());
                  if(g.length()<2)
                    g = "0"+g;
                  b = Integer.toHexString(color.getBlue());
                  if(b.length()<2)
                    b = "0"+b;
                  out.println("<td bgcolor=\"#"+ r + g + b + "\">"+seq.getCharAt(res)+"</td>");
                }
                else
                  out.println("<td>"+seq.getCharAt(res)+"</td>");

              }

         out.println("</tr>");

       }
       if(endRes<al.getWidth())
           out.println("<tr><td height=\"5\"></td></tr>");



     }

    out.println("</table>");
    out.println("</table>");



  }


}
